//----------------------------------------------------------------
// "A Game of Intelligent Risk" by David Russell Newman (drn101) 
// Class: Player
// Version: 1.0
// Last Modified: 17/12/2003
//----------------------------------------------------------------

import java.util.*;

/** 
 * This class creates the both the red and blue players that are 
 * generic.  This class can be parsed the input file of an 
 * artificially intelligent player to determine which moves the 
 * player will make.
 * @see java.lang.Object
 */
public class Player{
	int col, men, cells, toAllocateMen, toAllocateCells, initMen;
	float menReal;
	String type_file;
	Vector occupiedSqs;
	PlayerMoveGenerator pmg;
	RiskGameMove curMove;
	float insignificant, attDec, minAtt, maxAtt, reinforceRatio, defEmp, attEmp;
	boolean rankSignificance, attackTrack;
	String typeNumMen;
	
	/**
	* Constructs a new Player object.
	* @param colPar the colour of the player represented as an 
	* int, RED = 1, BLUE = 2.
	* @param menPar an int representing the number of men the 
	* player starts the game with.
	* @param cellsPar an int representing the number of squares
	* the player starts the game with
	*/
	public Player(int colPar, int menPar, int cellsPar){
		col = colPar;
		toAllocateMen = menPar;
		men = menPar;
		menReal = menPar;
		toAllocateCells = cellsPar;
		occupiedSqs = new Vector();
		type_file = new String();
		pmg = null;
		insignificant=0;
		rankSignificance=false;
		attDec=0;
		minAtt=0;
		maxAtt=1;
		typeNumMen = new String("RANDOM");
		reinforceRatio=0;
		attEmp=0;
		defEmp=0;
		attackTrack=false;
	}
	public void addSquare(Box boxPar){
		occupiedSqs.addElement(boxPar);
	}
	public boolean findSquare(BoxPosition boxPar){
		Enumeration enum1 = occupiedSqs.elements();
		while (enum1.hasMoreElements()){
			if (((Box)enum1.nextElement()).getBoxPosition().equals(boxPar)) return true;
		}
		return false;
	}
	public void removeSquare(BoxPosition boxPar){
		int i=0;
		boolean boxFound = false;
		Enumeration enum1 = occupiedSqs.elements();
		while (enum1.hasMoreElements() && !boxFound){
			if (((Box)enum1.nextElement()).getBoxPosition().equals(boxPar))boxFound=true;
			else i++;
		}
		if (boxFound) occupiedSqs.removeElementAt(i);
	}
	public float getNumMen(){
		float noMen = 0;
		Enumeration e = occupiedSqs.elements();
		while (e.hasMoreElements()){
			Box curBox = (Box)e.nextElement();
		 	noMen += curBox.getNum(col);
		}
		return noMen;
	}
	public void createPlayerMoveGenerator(int xMax, int yMax, boolean diagAdj){
		pmg = new PlayerMoveGenerator(col,type_file);
		Vector parsedSqs = new Vector(occupiedSqs);
		parsedSqs = randomiseSquares(parsedSqs);
		if(!type_file.equals("AI_PLAYER")){
			if (insignificant!=0){
				Vector oldSqs = new Vector(parsedSqs);
			 	parsedSqs = removeInsignificant(parsedSqs,getNumMen()*insignificant,col);
				if (parsedSqs.size()==0) parsedSqs=oldSqs;
			}
			if (rankSignificance) parsedSqs = rankSignificance(parsedSqs,col);
		}
		pmg.generateAllMoves(parsedSqs, xMax, yMax, diagAdj);
	}
	private static Vector rankSignificance(Vector sqsPar, int colPar){
		Box[] allBoxes = new Box[sqsPar.size()];
		sqsPar.toArray(allBoxes);
		Box[] tempBoxes = new Box[allBoxes.length];
		mergeSort(allBoxes, tempBoxes, 0, allBoxes.length-1, colPar);
		Vector orderedVec = new Vector();
		for (int i=0; i<allBoxes.length; i++) orderedVec.add(allBoxes[i]);
		return orderedVec;
	}

	private static void mergeSort(Box[] theBoxes, Box[] tmpBoxes, int left, int right, int colPar){
		if (left<right){
			int centre = (left+right)/2;
			mergeSort(theBoxes, tmpBoxes, left, centre, colPar);
			mergeSort(theBoxes, tmpBoxes, centre+1,right, colPar);
			merge(theBoxes, tmpBoxes, left, centre+1, right, colPar);
		}
	}

	private static void merge(Box[] theBoxes, Box[] tmpBoxes, int leftPos, int rightPos, int rightEnd, int colPar){
		int leftEnd = rightPos-1;
		int tmpPos = leftPos;
		int numElements = rightEnd-leftPos+1;

		while (leftPos<=leftEnd && rightPos<=rightEnd){
			if (theBoxes[leftPos].getNum(colPar)>theBoxes[rightPos].getNum(colPar)){
				tmpBoxes[tmpPos++] = theBoxes[leftPos++];
			}
			else{
				tmpBoxes[tmpPos++] = theBoxes[rightPos++];
			}
		}
		while (leftPos<=leftEnd){
			tmpBoxes[tmpPos++]=theBoxes[leftPos++];
		}
		while (rightPos<=rightEnd){
			tmpBoxes[tmpPos++]=theBoxes[rightPos++];
		}
		for (int i=0; i<numElements; i++, rightEnd--){
			theBoxes[rightEnd] = tmpBoxes[rightEnd];
		}
	}
	private static Vector removeInsignificant(Vector sqsPar, float insigMen, int colPar){
		int index = 0;
		Enumeration e = sqsPar.elements();
		while (e.hasMoreElements()){
			Box curBox = (Box)e.nextElement();
			if (curBox.getNum(colPar)<insigMen) sqsPar.remove(index);
			index++;
		}
		return sqsPar;
	}
	private static Vector randomiseSquares(Vector sqsPar){
		Vector randomisedVec = new Vector();
		while (sqsPar.size()>0){
			randomisedVec.add(sqsPar.remove((new Double(Math.floor(Math.random()*sqsPar.size()))).intValue()));
		}
		return randomisedVec;
	}
	public static void main(String args[]){
	/*	Vector aVec = new Vector();
		aVec.add("A");
		aVec.add("B");
		aVec.add("C");
		aVec.add("D");
		aVec.add("E");
		Vector secVec = randomiseSquares(aVec);
		Enumeration e = secVec.elements();
		while (e.hasMoreElements()){
			System.out.println((String)e.nextElement());
		}*/
		Player play = new Player(Box.RED,100,10);
		AIPlayerReader aipr = new AIPlayerReader(play,(String)args[0]);
		System.out.println("player = "+	play.getType()+";");
		System.out.println(play.getPlayerFile());
	}

	public RiskGameMove generateMove(float decOfMen, Grid gridPar, Vector oppSqsPar){
		try{
			pmg.generateMove(decOfMen, attDec, minAtt, maxAtt, typeNumMen, reinforceRatio, attEmp, defEmp, attackTrack, gridPar, oppSqsPar);
			return pmg.getMove();
		}
		catch(Exception e){System.out.println("Error in generate Move");}
		return null;
	}
	public boolean getShouldWithdraw(long numRedPar, long numBluePar){
		pmg = new PlayerMoveGenerator(col,type_file);
		return pmg.getShouldWithdraw(numRedPar, numBluePar, type_file);
	}
	public RiskGameMove getWithdraw(Vector allWDs){
		try{pmg = new PlayerMoveGenerator(col,type_file);
		return pmg.getWithdraw(allWDs, type_file);}
		catch(Exception e){System.out.println("Error in Player.getWithdraw");}
		return null;
	}						
	public RiskGameMove getMove(){
		return pmg.getMove();
	}
	public Vector getOccupiedSquares(){
		return occupiedSqs;
	}	
	public String getPlayerFile(){
		String pFile = 	"INSIGNIFICANT "+insignificant+";\r\nRANK_SIGNIFICANCE ";
		if (rankSignificance) pFile = pFile.concat("TRUE;\r\n");
		else pFile = pFile.concat("FALSE;\r\n");
		pFile = pFile.concat("ATTACK_DECIMAL "+attDec+";\r\nMINIMUM_ATTACK "+minAtt+";\r\nMAXIMUM_ATTACK "+maxAtt+";\r\nTYPE_NUM_MEN "+typeNumMen+";\r\nREINFORCE_RATIO "+reinforceRatio+";\r\nATTACK_EMPTY "+attEmp+";\r\nDEFEND_EMPTY "+defEmp+";");
		return pFile;
	}
	/**
	* Returns the colour of the player (red or blue) as an int 
	* value: RED = 1, BLUE =2.
	* @returns the colour of the player (red or blue) as an int 
	* value: RED = 1, BLUE =2.
	*/
	public int getCol(){
		return col;

	}

	/**
	* Returns an int representing the number of men the player 
	* has.
	* @returns an int representing the number of men the player 
	* has.
	*/
	public int getMen(){
		return men;
	}
	
	/**
	* Returns an float representing the number of men the player 
	* has.  (This can be something other than a whole number 
	* because the add men function does not add whole numbers of
	* men. 
	* @returns an int represnting the number of men the player 
	* has.
	*/
	public float getMenReal(){
		return menReal;
	}

	/**
	* Sets the number of cells/squares the player occupies
	* @param cellsPar the number of cells/squares the player 
	* is to be set to occupy.
	*/
	public void setNumCells(int cellsPar){
		cells = cellsPar;
	}

	/**
	* Sets the number of men the player has.
	* @param menPar an int representing the number of men the 
	* player is to be set to have.
	*/
	public void setMen(int menPar){
		men = menPar;
	}

	/**
	* Returns the number of men the player has that still have
	* to be allocated to a square.
	* @returns the number of men the player has that still have
	* to be allocated to a square.
	*/
	public int getToAllocateMen(){
		return toAllocateMen;
	}

	/**
	* Sets the number of men that still have to be allocated to
	* a square for this player.
	* @param toAllocateMenPar an int representing that is going 
	* to be set as the number of men that still need to be 
	* allocated to a square for this player.
	*/ 
	public void setToAllocateMen(int toAllocateMenPar){
		toAllocateMen = toAllocateMenPar;
	}
	
	/** 
	* Gets the number of cells/squares that still need to be 
	* allocated for the player.
	* @returns at int reprsenting the number of cells/squares 
	* that still need to be allocated for the player. 
	*/
	public int getToAllocateCells(){
		return toAllocateCells;
	}

	/**
	* Sets the type of player, whether a human player or an AI
	* player.  If the player is to be set as human, the value 
	* parsed is the String "HUMAN", if it is an AI player the 
	* pathname/filename of the AI player input file is parsed.
	* @param tfPar the String "HUMAN" or the pathname/filename
	* of the AI player input file.
	*/ 
	public void setType(String tfPar){
		type_file = tfPar;
		if ((!type_file.equals("HUMAN")) && (!type_file.equals("AI_PLAYER"))){
		//	AIPlayerReader aipr = new AIPlayerReader(this,type_file);
		}
	}

	/**
	* Gets the type of player, whether a human player or an AI
	* player.  If the player is to be human, the value returned
	* is the String "HUMAN", if it is an AI player the 
	* pathname/filename of the AI player input file is returned.
	* @returns the String "HUMAN" or the pathname/filename
	* of the AI player input file.
	*/ 
	public String getType(){
		return type_file;
	}
	public Vector getSquaresVec(){
		return occupiedSqs;
	}
	public int getNoOccupiedSquares(){
		return occupiedSqs.size();
	}
	public void setInsignificant(float insigPar){
		insignificant = insigPar;
	}
	public float getInsignificant(){
		return insignificant;
	}
	public boolean getRankSignificance(){
		return rankSignificance;
	}
	public void setRankSignificance(boolean rsPar){
		rankSignificance = rsPar;
	}
	public void setAttackDecimal(float adPar){
		attDec = adPar;
	}
	public float getAttackDecimal(){
		return attDec;
	} 	
	public void setMinimumAttack(float minAttPar){
		minAtt = minAttPar;
	}
	public float getMinimumAttack(){
		return minAtt;
	} 
	public void setMaximumAttack(float maxAttPar){
		maxAtt= maxAttPar;
	}
	public float getMaximumAttack(){
		return maxAtt;
	}
	public void setTypeNumMen(String typeNumMenPar){
		typeNumMen = typeNumMenPar;
	}
	public String getTypeNumMen(){
		return typeNumMen;
	}
	public void setReinforceRatio(float rrPar){
		reinforceRatio = rrPar;
	}
	public float getReinforceRatio(){
		return reinforceRatio;
	}
	public void setAttackEmpty(float attEmpPar){
		attEmp=attEmpPar;
	}
	public float getAttackEmpty(){
		return attEmp;
	}
	public void setDefendEmpty(float defEmpPar){
		defEmp=defEmpPar;
	}
	public float getDefendEmpty(){
		return defEmp;
	}
	public void setAttackTracking(boolean atPar){
		attackTrack = atPar;
	}
	public boolean getAttackTracking(){
		return attackTrack;
	}
}