//----------------------------------------------------------------
// "A Game of Intelligent Risk" by David Russell Newman (drn101) 
// Class: PlayerMoveGenerator
// Version: 1.0
// Last Modified: 28/01/2004
//----------------------------------------------------------------

import java.util.*;
import java.io.*;
import javax.swing.*;

public class PlayerMoveGenerator{
	int forPlayer, xMax, yMax;
	boolean diagAdj;
	RiskGameMove decidedMove;
	Vector allMoves, allSquares;
	String fileType;
	public PlayerMoveGenerator(int colPar, String fileTypePar){
		forPlayer = colPar;
		fileType=fileTypePar;
	}
	public void generateAllMoves(Vector allSquaresPar, int xMaxPar, int yMaxPar, boolean diagAdjPar){
		BoxPosition toBP, curBP;
		allSquares = allSquaresPar;
		diagAdj = diagAdjPar;
		xMax = xMaxPar;
		yMax = yMaxPar;
		Enumeration enum1 = allSquares.elements();
		allMoves = new Vector();
		while (enum1.hasMoreElements()){
			Vector amfSqs = new Vector();
			curBP = ((Box)enum1.nextElement()).getBoxPosition();
			if (!(curBP.getHoriz()==xMax-1)) amfSqs.addElement(new RiskGameMove(curBP,new BoxPosition(curBP.getHoriz()+1, curBP.getVert())));
			if (!(curBP.getHoriz()==0)) amfSqs.addElement(new RiskGameMove(curBP,new BoxPosition(curBP.getHoriz()-1, curBP.getVert())));
			if (!(curBP.getVert()==yMax-1)) amfSqs.addElement(new RiskGameMove(curBP,new BoxPosition(curBP.getHoriz(), curBP.getVert()+1)));
			if (!(curBP.getVert()==0)) amfSqs.addElement(new RiskGameMove(curBP,new BoxPosition(curBP.getHoriz(), curBP.getVert()-1)));
			if (diagAdj){
				if (!(curBP.getHoriz()==xMax-1 || curBP.getVert()==yMax-1)) amfSqs.addElement(new RiskGameMove(curBP,new BoxPosition(curBP.getHoriz()+1, curBP.getVert()+1)));
				if (!(curBP.getHoriz()==xMax-1 || curBP.getVert()==0)) amfSqs.addElement(new RiskGameMove(curBP,new BoxPosition(curBP.getHoriz()+1, curBP.getVert()-1)));
				if (!(curBP.getHoriz()==0 || curBP.getVert()==yMax-1)) amfSqs.addElement(new RiskGameMove(curBP,new BoxPosition(curBP.getHoriz()-1, curBP.getVert()+1)));
				if (!(curBP.getHoriz()==0 || curBP.getVert()==0)) amfSqs.addElement(new RiskGameMove(curBP,new BoxPosition(curBP.getHoriz()-1, curBP.getVert()-1)));
			}
			allMoves.addAll(new Vector(randomiseMoves(amfSqs)));
		}		
	}
	private static Vector randomiseMoves(Vector movesPar){
		Vector randomisedVec = new Vector();
		while (movesPar.size()>0){
			RiskGameMove rgm = (RiskGameMove)movesPar.remove((new Double(Math.floor(Math.random()*movesPar.size()))).intValue());
	//		System.out.println("The toBox is "+rgm.getToBox().getHoriz()+", "+rgm.getToBox().getVert());
			randomisedVec.add(rgm);
		}
		return randomisedVec;
	}
	public void generateMove(float decOfMen, float attDecPar, float minAttPar, float maxAttPar, String typeNumMenPar, float reinforceRatio, float attEmpPar, float defEmpPar, boolean attTraPar, Grid gridPar, Vector oppSqsPar){
		decidedMove=null;
		if(fileType.equals("AI_PLAYER")){
		//	System.out.println("Start of generateMove");
		//	System.out.println("allMoves.size() = "+allMoves.size());
			decidedMove = (RiskGameMove)allMoves.elementAt((new Double(Math.floor(Math.random()*allMoves.size()))).intValue());
			Box curBox = gridPar.getBox(decidedMove.getFromBox());
			if (findBox(decidedMove.getFromBox(),allSquares)) decidedMove.setNoMen((new Double(Math.ceil(Math.random()*curBox.getNum(forPlayer)))).intValue());
			else{				
				System.out.println("Box not found");
				decidedMove=null;	 
			}
		}
		else{
			int curPos=0;
			try{ 
			int oppSqsSize=0;
			int empSqsSize=0;
		//	System.out.println("Start of generateMove");
		//	System.out.println("allMoves.size() = "+allMoves.size());
			if (attDecPar!=0){
				curPos=1;
				Vector yourSqs = new Vector();
				Vector emptySqs = new Vector();
				Vector oppSqs = new Vector();
				curPos=2;
				while (allMoves.size()>0){
					curPos=3;
					RiskGameMove rgm = (RiskGameMove)allMoves.remove(0);
					if (gridPar.getBox(rgm.getToBox()).getNum(forPlayer)>0) yourSqs.add(rgm);
					else if ((forPlayer==Box.BLUE && gridPar.getBox(rgm.getToBox()).getNum(Box.RED)>0)||(forPlayer==Box.RED && gridPar.getBox(rgm.getToBox()).getNum(Box.BLUE)>0)) oppSqs.add(rgm);
					else emptySqs.add(rgm);
				}
				curPos=4;
			//	System.out.println("Decimal of men of player = "+decOfMen);
				if (attDecPar>=decOfMen){
					curPos=5;
					allMoves.addAll(emptySqs);
					allMoves.addAll(yourSqs);
					allMoves.addAll(oppSqs);
				}
				else{
					curPos=6;
					allMoves.addAll(oppSqs);
					allMoves.addAll(emptySqs);
					allMoves.addAll(yourSqs);
				}
				curPos=7;	
				oppSqsSize = oppSqs.size();
				empSqsSize = emptySqs.size();
				curPos=8;
			}
			if (minAttPar!=0 && attDecPar<decOfMen){
				curPos=9;
				int j=0;
				while (decidedMove==null && j<oppSqsSize){
					curPos=10;
					RiskGameMove curMove = (RiskGameMove)allMoves.elementAt(j);
					curPos=11;
					long yourMen = gridPar.getBox(curMove.getFromBox()).getNum(forPlayer);
					if (yourMen>0){
						curPos=12;
						long theirMen=0;
						if (forPlayer==Box.RED) theirMen = gridPar.getBox(curMove.getToBox()).getNum(Box.BLUE);
						else theirMen = gridPar.getBox(curMove.getToBox()).getNum(Box.RED);
						curPos=13;
				//		System.out.println("yourMen = "+yourMen+", theirMen*minAttPar = "+theirMen*minAttPar);
						int min = (new Double(Math.ceil(minAttPar*theirMen))).intValue();
						curPos=14;
						int max = (new Double(Math.floor(maxAttPar*yourMen))).intValue();
						curPos=15;
						if (max>=min) decidedMove = curMove;
						curPos=16;
					}
					j++;
				}
				curPos=17;
				if (decidedMove==null){
					curPos=18;
					if (oppSqsSize==0){}
						
					if (allMoves.size()==oppSqsSize) decidedMove = (RiskGameMove)allMoves.elementAt(0);
					else{
						j=0;
						curPos=18;
						if (reinforceRatio>0){
							//System.out.println("Used Reinforce ratio");
							curPos=19;
							while (decidedMove==null && j<oppSqsSize){			
								RiskGameMove curMove = (RiskGameMove)allMoves.elementAt(j);
								curPos=20;
								Vector moveSqs = findSquares(curMove.getFromBox(),xMax,yMax,diagAdj);
								curPos=21;
								while (moveSqs.size()>0 && decidedMove==null){
									decidedMove = (RiskGameMove)moveSqs.remove(0);
									if (!findBox(decidedMove.getFromBox(),allSquares)) decidedMove=null;
									curPos=22;
								}
								j++;
							}
							curPos=23;
							if (decidedMove!=null){
							//	System.out.println("Move found");
								 decidedMove.setNoMen((new Double(gridPar.getBox(decidedMove.getFromBox()).getNum(forPlayer)*reinforceRatio)).intValue());
							}
							curPos=24;
						}
					}
				}
			//	if (decidedMove==null) decidedMove = (RiskGameMove)allMoves.elementAt(0);
			//	curPos=18;
			}
			curPos=19;
			if (decidedMove==null && attDecPar<decOfMen){
				if (oppSqsSize==allMoves.size()) decidedMove = (RiskGameMove)allMoves.elementAt(0);
				else decidedMove = (RiskGameMove)allMoves.elementAt(oppSqsSize);
			}
			else if (decidedMove==null) decidedMove = (RiskGameMove)allMoves.elementAt(0);
			curPos=20;
			boolean boxFound=false;
			Box curBox = null;
			BoxPosition fromBP = decidedMove.getFromBox();
			if (!findBox(fromBP, allSquares)){
				curPos=25;
				BoxPosition toBP = decidedMove.getToBox();
				System.out.println("Box not found at X = "+fromBP.getHoriz()+", Y = "+fromBP.getVert()+", toBox.x = "+toBP.getHoriz()+", y = "+toBP.getVert());
				decidedMove=null;
				curPos=26;
			}
			else if (decidedMove.getNoMen()==0){ 
				curPos=27;
				long yourMen = gridPar.getBox(decidedMove.getFromBox()).getNum(forPlayer);
				if (yourMen==0) System.out.println("You cannot move men from a square with no men in");
				curPos=28;
				long theirMen=0;
				if (forPlayer==Box.RED) theirMen = gridPar.getBox(decidedMove.getToBox()).getNum(Box.BLUE);
				else theirMen = gridPar.getBox(decidedMove.getToBox()).getNum(Box.RED);
				curPos=29;
				int min = (new Double(Math.ceil(minAttPar*theirMen))).intValue();
				curPos=30;
				int max = (new Double(Math.floor(maxAttPar*yourMen))).intValue();
				curPos=31;
				if (min==0){
					if (gridPar.getBox(decidedMove.getToBox()).getNum(forPlayer)==0){
						if (decOfMen>attDecPar && attEmpPar>0){
							decidedMove.setNoMen((new Double(Math.floor(yourMen*attEmpPar))).intValue());
							//System.out.println("ATTACK_EMPTY parameter used.");
						}
						else if (decOfMen<=attDecPar && defEmpPar>0){
							decidedMove.setNoMen((new Double(Math.floor(yourMen*defEmpPar))).intValue());
							//System.out.println("DEFEND_EMPTY parameter used.");
						}
						else decidedMove.setNoMen((new Double(Math.ceil(Math.random()*yourMen))).intValue());
					}
					else decidedMove.setNoMen((new Double(Math.ceil(Math.random()*yourMen))).intValue());
				}
				else if (max>=min){
					if (typeNumMenPar.equals("MAX")) decidedMove.setNoMen(max);
					else if (typeNumMenPar.equals("MIN")) decidedMove.setNoMen(min);
					else if (typeNumMenPar.equals("AVERAGE")) decidedMove.setNoMen((new Double(Math.ceil((max+min)/2))).intValue());
					else decidedMove.setNoMen((new Double(Math.ceil(Math.random()*(max-min)))).intValue()+min);
				}
				else if (max<min){
					if (typeNumMenPar.equals("MAX")||typeNumMenPar.equals("MIN")||typeNumMenPar.equals("AVERAGE")) decidedMove.setNoMen(max);
					else decidedMove.setNoMen((new Double(Math.ceil(Math.random()*max))).intValue());
				}
				curPos=32;
			}
			}catch (Exception e){System.out.println("Error at curPos = "+curPos+", allMoves size = "+allMoves.size()+"allSquares size = "+allSquares.size()+", colour = "+forPlayer);}	
		}	
	}
	public static boolean findBox(BoxPosition fromBPPar, Vector allSqsPar){
		Enumeration enum1 = allSqsPar.elements();
		while (enum1.hasMoreElements()){
			Box curBox = (Box)enum1.nextElement();
			if (fromBPPar.equals(curBox.getBoxPosition())) return true;
		}
		return false;
	}

	private static Vector findSquares(BoxPosition curBP, int xPar, int yPar, boolean diagPar){
		Vector amfSqs = new Vector(); 
		if (!(curBP.getHoriz()==xPar-1)) amfSqs.addElement(new RiskGameMove(new BoxPosition(curBP.getHoriz()+1, curBP.getVert()),curBP));
		if (!(curBP.getHoriz()==0)) amfSqs.addElement(new RiskGameMove(new BoxPosition(curBP.getHoriz()-1, curBP.getVert()),curBP));
		if (!(curBP.getVert()==yPar-1)) amfSqs.addElement(new RiskGameMove(new BoxPosition(curBP.getHoriz(), curBP.getVert()+1),curBP));
		if (!(curBP.getVert()==0)) amfSqs.addElement(new RiskGameMove(new BoxPosition(curBP.getHoriz(), curBP.getVert()-1),curBP));
		if (diagPar){
			if (!(curBP.getHoriz()==xPar-1 || curBP.getVert()==yPar-1)) amfSqs.addElement(new RiskGameMove(new BoxPosition(curBP.getHoriz()+1, curBP.getVert()+1),curBP));
			if (!(curBP.getHoriz()==xPar-1 || curBP.getVert()==0)) amfSqs.addElement(new RiskGameMove(new BoxPosition(curBP.getHoriz()+1, curBP.getVert()-1),curBP));
			if (!(curBP.getHoriz()==0 || curBP.getVert()==yPar-1)) amfSqs.addElement(new RiskGameMove(new BoxPosition(curBP.getHoriz()-1, curBP.getVert()+1),curBP));
			if (!(curBP.getHoriz()==0 || curBP.getVert()==0)) amfSqs.addElement(new RiskGameMove(new BoxPosition(curBP.getHoriz()-1, curBP.getVert()-1),curBP));
		}
		Vector newVec = new Vector(randomiseMoves(amfSqs));
		return newVec;
	}
		
	private static Vector rankAttackRatios(Vector sqsPar, int colPar, Grid gridPar){
		RiskGameMove[] allAttacks = new RiskGameMove[sqsPar.size()];
		sqsPar.toArray(allAttacks);
		RiskGameMove[] tempAttacks = new RiskGameMove[allAttacks.length];
		mergeSort(allAttacks, tempAttacks, 0, allAttacks.length-1, colPar, gridPar);
		Vector orderedVec = new Vector();
		for (int i=0; i<allAttacks.length; i++) orderedVec.add(allAttacks[i]);
		return orderedVec;
	}

	private static void mergeSort(RiskGameMove[] theAttacks, RiskGameMove[] tmpAttacks, int left, int right, int colPar, Grid gridPar){
		if (left<right){
			int centre = (left+right)/2;
			mergeSort(theAttacks, tmpAttacks, left, centre, colPar, gridPar);
			mergeSort(theAttacks, tmpAttacks, centre+1,right, colPar, gridPar);
			merge(theAttacks, tmpAttacks, left, centre+1, right, colPar, gridPar);
		}
	}

	private static void merge(RiskGameMove[] theAttacks, RiskGameMove[] tmpAttacks, int leftPos, int rightPos, int rightEnd, int colPar, Grid gridPar){
		int leftEnd = rightPos-1;
		int tmpPos = leftPos;
		int numElements = rightEnd-leftPos+1;

		while (leftPos<=leftEnd && rightPos<=rightEnd){
			float leftPosRatio = theAttacks[leftPos].getAttackRatio(colPar, gridPar);
			float rightPosRatio = theAttacks[rightPos].getAttackRatio(colPar, gridPar);
			if (leftPosRatio>rightPosRatio){
				tmpAttacks[tmpPos++] = theAttacks[leftPos++];
			}
			else{
				tmpAttacks[tmpPos++] = theAttacks[rightPos++];
			}
		}
		while (leftPos<=leftEnd){
			tmpAttacks[tmpPos++]=theAttacks[leftPos++];
		}
		while (rightPos<=rightEnd){
			tmpAttacks[tmpPos++]=theAttacks[rightPos++];
		}
		for (int i=0; i<numElements; i++, rightEnd--){
			theAttacks[rightEnd] = tmpAttacks[rightEnd];
		}
	}
	public RiskGameMove getMove(){
		return decidedMove;
	}
	public boolean getShouldWithdraw(long numRedPar, long numBluePar, String typeFilePar){
		int curPlace=0;
		if (typeFilePar.equals("AI_PLAYER")){
			long numTot = numRedPar+numBluePar;
			long randNum = (new Double(Math.ceil(Math.random()*numTot))).longValue();
			if (numTot==randNum) return true;
			else return false;
		}
		return false;
	}
	public RiskGameMove getWithdraw(Vector allRetreats, String typeFilePar){
		int curPos = 0;
		try{
		if (typeFilePar.equals("AI_PLAYER")){
			curPos = 1;
			if (allRetreats.size()==0) return null;
			RiskGameMove rgm = (RiskGameMove)allRetreats.elementAt((new Double(Math.floor(Math.random()*allRetreats.size()))).intValue());
			curPos =2;
			return rgm;
		}
		else return null;
		}catch(Exception e){System.out.println("Error in pmg.getWithdraw at curPos = "+curPos);}
		return null;
	}
}
		
	
	