//----------------------------------------------------------------
// "A Game of Intelligent Risk" by David Russell Newman (drn101) 
// Class: RiskGameReader
// Version: 1.0
// Last Modified: 17/12/2003
//----------------------------------------------------------------

import java.io.*;
import java.util.*;

public class RiskGameReader{
	private RiskGame theGame;	

	public RiskGameReader(File gameFile) throws IOException{
		Vector myTokens = new Vector();
		BufferedReader stream = new BufferedReader(new InputStreamReader(
			new FileInputStream(gameFile)));
		while (stream.ready()){
			String curStr = stream.readLine();
			StringTokenizer st = new StringTokenizer(curStr, ";");
			while (st.hasMoreTokens()){
				myTokens.addElement(st.nextToken());
			}
		}
		Enumeration rules = myTokens.elements();
		theGame = new RiskGame();
		try{
			setGrid((String)rules.nextElement(), (String)rules.nextElement());
			setMen((String)rules.nextElement());
			setSquares((String)rules.nextElement(), (String)rules.nextElement());
			setMenAllocation((String)rules.nextElement());
			setMenAdded((String)rules.nextElement());
			setAdjacent((String)rules.nextElement());
			setWithdraw((String)rules.nextElement(), (String)rules.nextElement());
			setResult((String)rules.nextElement());
			setDraws((String)rules.nextElement());
			setScoreMax((String)rules.nextElement());
			setWeighted((String)rules.nextElement());
			setWinner((String)rules.nextElement());
			setNoTurns((String)rules.nextElement());
			setPoints((String)rules.nextElement(), (String)rules.nextElement());
			setMovesPerTurn((String)rules.nextElement());
			setRedPlayer((String)rules.nextElement());
			setBluePlayer((String)rules.nextElement());
			System.out.println("Finished");
		}
		catch (Exception e){System.out.println(e.getMessage());}
	}
	
	private void setGrid(String xPar, String yPar){
		theGame.setGridX(Integer.parseInt(xPar.substring(7)));
		theGame.setGridY(Integer.parseInt(yPar.substring(7)));
		theGame.setGrid();
		System.out.println("X = "+theGame.getGridX()+"; Y = "+theGame.getGridY()+";");
	}
	private void setMen(String menPar){
		theGame.setNoMen(Integer.parseInt(menPar.substring(15)));
		System.out.println("Men = "+theGame.getNoMen()+";");
	}
	private void setSquares(String noSqPar, String sqAllocPar) throws Exception{
		theGame.setNoInitSquares(Integer.parseInt(noSqPar.substring(14)));
		System.out.println("Number of Squares = "+theGame.getNoInitSquares()+";");
		String squareAlloc = sqAllocPar.substring(18);
		if (squareAlloc.equals("RANDOM"))
			theGame.setSquareAllocStatus(RiskGame.RANDOM);
		else if (squareAlloc.equals("CHOSEN"))
			theGame.setSquareAllocStatus(RiskGame.CHOSEN);
		else if (squareAlloc.equals("NONE"))
			theGame.setSquareAllocStatus(RiskGame.NONE);
		else if (squareAlloc.equals("FROM_FILE"))
			theGame.setSquareAllocStatus(RiskGame.FROM_FILE);
		else throw new Exception("Game file does not specify a valid form of square allocation.");	
		if (theGame.getSquareAllocStatus() == RiskGame.RANDOM)
			System.out.println("Square Allocation = RANDOM;");
		else if (theGame.getSquareAllocStatus() == RiskGame.CHOSEN)
			System.out.println("Square Allocation = CHOSEN;");
		else if (theGame.getSquareAllocStatus() == RiskGame.NONE)
			System.out.println("Square Allocation = NONE;");
		else if (theGame.getSquareAllocStatus() == RiskGame.FROM_FILE)
			System.out.println("Square Allocation = FROM_FILE;");
	}
	private void setMenAllocation(String menAllocPar) throws Exception{
		String menAlloc = menAllocPar.substring(15);
		if (menAlloc.equals("RANDOM"))
			theGame.setManAllocStatus(RiskGame.RANDOM);
		else if (menAlloc.equals("CHOSEN"))
			theGame.setManAllocStatus(RiskGame.CHOSEN);
		else if (menAlloc.equals("SQUARE_SELECT"))
			theGame.setManAllocStatus(RiskGame.SQUARE_SELECT);
		else if (menAlloc.equals("FROM_FILE"))
			theGame.setManAllocStatus(RiskGame.FROM_FILE);
		else throw new Exception("Game file does not specify a valid form of men allocation.");	
		if (theGame.getManAllocStatus()==RiskGame.RANDOM)
			System.out.println("Man Allocation = RANDOM;");
		else if (theGame.getManAllocStatus()==RiskGame.CHOSEN)
			System.out.println("Man Allocation = CHOSEN;");
		else if (theGame.getManAllocStatus()==RiskGame.SQUARE_SELECT)
			System.out.println("Man Allocation = SQUARE_SELECT;");
		else if (theGame.getManAllocStatus()==RiskGame.FROM_FILE)
			System.out.println("Man Allocation = FROM_FILE;");
	}

	private void setMenAdded(String menAddedPar){
		theGame.setMenAddedFunction(menAddedPar.substring(10));
		System.out.println("Men Adding Function = "+theGame.getMenAddedFunction()+";");  		
	}
	private void setAdjacent(String adjacentPar){
		theGame.setDiagonalSquaresAdjacent(adjacentPar.substring(26).equals("TRUE"));
		System.out.println("Diagonal Squares Adjacent = "+theGame.getDiagonalSquaresAdjacent()+";");
	}
	private void setWithdraw(String defendWDPar, String attackWDPar){
		theGame.setDefenderWithdraw(defendWDPar.substring(18).equals("TRUE"));
		System.out.println("Defender Can Withdraw = "+theGame.getDefenderWithdraw()+";");
		theGame.setAttackerWithdraw(attackWDPar.substring(18).equals("TRUE"));
		System.out.println("Attacker Can Withdraw = "+theGame.getAttackerWithdraw()+";");	
	}
	private void setResult(String resultPar) throws Exception{
		String result = resultPar.substring(13);
		if (result.equals("ONE_MAN"))
			theGame.setScoreResultStatus(RiskGame.ONE_MAN);
		else if (result.equals("BATTLE"))
			theGame.setScoreResultStatus(RiskGame.BATTLE);
		else throw new Exception("Game file does not specify a valid form of score result.");
		if (theGame.getScoreResultStatus()==RiskGame.ONE_MAN)
			System.out.println("Score Result = ONE_MAN;");
		else if (theGame.getScoreResultStatus()==RiskGame.BATTLE)
			System.out.println("Score Result = BATTLE;");
		
	}
	private void setDraws(String drawsPar) throws Exception{
		String draws = drawsPar.substring(6);
		if (draws.equals("DEFENDER_WINS"))
			theGame.setDraws(RiskGame.DEFENDER_WINS);
		else if (draws.equals("ATTACKER_WINS"))
			theGame.setDraws(RiskGame.ATTACKER_WINS);
		else if (draws.equals("REROLLED"))
			theGame.setDraws(RiskGame.REROLLED);
		else throw new Exception("Game file does not specify a valid form of draw resolution.");
		if (theGame.getDraws()==RiskGame.DEFENDER_WINS)
			System.out.println("Draw = DEFENDER_WINS;");
		else if (theGame.getDraws()==RiskGame.ATTACKER_WINS)
			System.out.println("Draw = ATTACKER_WINS;");
		else if (theGame.getDraws()==RiskGame.REROLLED)
			System.out.println("Draw = REROLLED;");
	}
	private void setScoreMax(String scoreMaxPar) throws Exception{
		try{
			theGame.setScoreMaxInt(Integer.parseInt(scoreMaxPar.substring(10)));
			if (theGame.getScoreMaxInt()<2){
				throw new Exception("Game file must specify a score maximum greater than 1.");
			}
			System.out.println("Score maximum = "+theGame.getScoreMaxInt()+";"); 	
		}
		catch (NumberFormatException e){
			String scoreMax = scoreMaxPar.substring(10);
			if (scoreMax.equals("NO_OF_MEN_IN_BATTLE"))
				theGame.setScoreMaxVariable("NO_OF_MEN_IN_BATTLE");
			else if (scoreMax.equals("NO_OF_SQUARES_ON_GRID"))
				theGame.setScoreMaxVariable("NO_OF_SQUARES_ON_GRID");
			else throw new Exception("Game file must specify a valid type of score maximum.");
			if (theGame.getScoreMaxVariable().equals("NO_OF_MEN_IN_BATTLE"))
				System.out.println("Score maximum = NO_OF_MEN_IN_BATTLE;"); 
			else if (theGame.getScoreMaxVariable().equals("NO_OF_SQUARES_ON_GRID"))
				System.out.println("Score maximum = NO_OF_SQUARES_ON_GRID;"); 
		}
	}
	private void setWeighted(String weightedPar){
		theGame.setWeighted(weightedPar.substring(9).equals("TRUE"));
		System.out.println("Weighted = "+theGame.getWeighted()+";");
	}
	private void setWinner(String winnerPar) throws Exception{
		String winner = winnerPar.substring(7);
		if (winner.equals("LAST_PLAYER"))
			theGame.setWinnerStatus(RiskGame.LAST_PLAYER);
		else if (winner.equals("SCORED"))
			theGame.setWinnerStatus(RiskGame.SCORED);
		else throw new Exception("Game file must specify a valid type of winning.");
		if (theGame.getWinnerStatus()==RiskGame.LAST_PLAYER)
			System.out.println("Winner = LAST_PLAYER;"); 
		else if (theGame.getWinnerStatus()==RiskGame.SCORED)
			System.out.println("Winner = SCORED;"); 
	}
	private void setNoTurns(String noTurnsPar){
		theGame.setNoTurns(Integer.parseInt(noTurnsPar.substring(12)));
		System.out.println("Number of turns each = "+theGame.getNoTurns()+";"); 
	}
	private void setPoints(String manPointsPar, String squarePointsPar){
		theGame.setPointsPerMan(Integer.parseInt(manPointsPar.substring(15)));
		System.out.println("Points per man = "+theGame.getPointsPerMan()+";"); 
		theGame.setPointsPerSquare(Integer.parseInt(squarePointsPar.substring(18)));
		System.out.println("Points per square = "+theGame.getPointsPerSquare()+";"); 
	}
	private void setMovesPerTurn(String movesPerTurnPar){
		theGame.setMovesPerTurn(Integer.parseInt(movesPerTurnPar.substring(15)));
		System.out.println("Moves per turn = "+theGame.getMovesPerTurn()+";"); 
	}
	private void setRedPlayer(String playerTypePar){
		String rpType = playerTypePar.substring(11);
		theGame.getRedPlayer().setType(rpType);	
		if (!(rpType.equals("HUMAN") || rpType.equals("AI_PLAYER"))){
			AIPlayerReader aipr = new AIPlayerReader(theGame.getRedPlayer(),rpType);
		}
		System.out.println("Red player = "+	theGame.getRedPlayer().getType()+";");
	}
	private void setBluePlayer(String playerTypePar){
		String bpType = playerTypePar.substring(12);
		theGame.getBluePlayer().setType(bpType);	
		if (!(bpType.equals("HUMAN") || bpType.equals("AI_PLAYER"))){
			AIPlayerReader aipr = new AIPlayerReader(theGame.getBluePlayer(),bpType);
		}
		System.out.println("Blue player = "+	theGame.getBluePlayer().getType()+";");
	}
	public RiskGame getRiskGame(){
		return theGame;
	}			
	public static void main(String[] argv) throws IOException{
		RiskGameReader theGameReader = new RiskGameReader(new File("game1.rg")); 
		//System.out.println(theGame.getGameText());
	}
}
